# EPICS IOC Startup Script generated by https://github.com/epics-containers/ibek

cd "{{ _global.get_env('IOC') }}"
{% if env_var_elements %}
{{ env_var_elements}}
{% endif -%}

dbLoadDatabase dbd/ioc.dbd
ioc_registerRecordDeviceDriver pdbbase

{{ script_elements }}
dbLoadRecords {{ _global.get_env('RUNTIME_DIR') }}/ioc.db
iocInit
{% if post_ioc_init_elements %}
{{ post_ioc_init_elements }}
{%- endif -%}
